package cn.wise.sc.energy.power.plant.business.service;

import cn.wise.sc.energy.power.plant.business.domain.DeviceInfo;
import cn.wise.sc.energy.power.plant.business.domain.vo.DeviceInfoVo;
import cn.wise.sc.energy.power.plant.business.domain.vo.EntityVo;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @description: 设备服务层接口
 * @author: qh
 * @create: 2020-07-14 16:35
 **/
@Service
public interface IDeviceInfoService extends IBaseService<String, DeviceInfo> {
    /**
     * 获取特定plantId的所有设备信息
     *
     * @param plantId 电厂编号
     * @return List<EntityVo>
     */
    List<EntityVo> getDeviceInfoByPlantCode(String plantId);

    /**
     * 获取特定plantId的所有设备和部件信息
     * @param plantId 电厂编号
     * @return List<EntityVo>
     */
    List<DeviceInfoVo> getDeviceAndUnitByPlantCode(String plantId);

    /**
     * 获取设备下的所有消息
     * @param plantId 电厂id
     * @return EntityVo
     */
    List<DeviceInfoVo> getDeviceUnitAndCharacterPlantCode(String plantId);

    /**
     * 获取功率和有功
     * @param deviceId 机组id
     * @return 最新设备的有功和功率
     */
    Map<String,Integer> getDevicePowerAndActive(String deviceId);
}
