package cn.wise.sc.energy.power.plant.business.task;

import cn.wise.sc.energy.power.plant.business.bean.DataPower;
import cn.wise.sc.energy.power.plant.business.domain.vo.CharacterParamInfoVo;
import cn.wise.sc.energy.power.plant.business.opentsdb.OpentsdbOkHttpClient;
import cn.wise.sc.energy.power.plant.business.service.ICharacterParamService;
import cn.wise.sc.energy.power.plant.business.service.IEventInfoService;
import cn.wise.sc.energy.power.plant.business.task.schedule.ScheduleTask;
import com.alibaba.fastjson.JSON;
import net.opentsdb.client.api.query.response.QueryResponse;
import net.opentsdb.client.bean.QueryResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author neo.shu
 * @since 2020/9/17 17:30
 */
@Service(value = "tendencyTask")
public class TendencyTask extends ScheduleTask {

    @Autowired
    ICharacterParamService iCharacterParamService;
    @Autowired
    IEventInfoService iEventInfoService;

    @Autowired
    TaskCacheDataService taskCacheDataService;

    public TendencyTask() {
        super(UUID.randomUUID().toString());
    }

    @Override
    public void run() {
        ConcurrentHashMap<String, TendencyWebSocket> webSocketMap =
                TendencyWebSocket.webSocketMap;
        ConcurrentHashMap<String, String> deviceMap = TendencyWebSocket.deviceMap;
        if (webSocketMap.size() == 0 || deviceMap.size() == 0) {
            return;
        }
        //给每个deviceid分组，然后统一查询，进行推送
        Map<String, List<Map.Entry<String, String>>> groupMap = deviceMap.entrySet().stream().collect(Collectors.groupingBy(c -> c.getValue()));

        //向每个webSocket推送系统实时数据
        for (String kkscode : groupMap.keySet()) {
            List<String> kksCodeList = new ArrayList<>();
            kksCodeList.add(kkscode);
            Long lastTime = OpentsdbOkHttpClient.queryLastBykks(kksCodeList);

            QueryResponse response = iCharacterParamService
                    .getRealTimeDataByKksCode(kksCodeList, (lastTime - 1) + "", lastTime + "", "");

            //组装数据结构
            List<Object> xAxis = response.getResults().get(0).getDps().keySet().stream().map(item -> item.toString()).collect(Collectors.toList());
            Map<String, DataPower> map = new HashMap<>();
            for (QueryResult queryResult : response.getResults()) {
                DataPower.PowerPoints powerPoints = new DataPower.PowerPoints();
                powerPoints.setKksCode(queryResult.getMetric());
                List<String> finalList = new ArrayList<>();
                queryResult.getDps().values().stream().forEach(item -> {
                    finalList.add(item.toString());
                });
                powerPoints.setValue(finalList);
                DataPower dataPower = map.getOrDefault("data", new DataPower());
                dataPower.setXAxis(xAxis);
                dataPower.getDataList().add(powerPoints);
                map.putIfAbsent("data", dataPower);
            }
            //左上图
            try {
                List<Map.Entry<String, String>> deviceGroupMap = groupMap.get(kkscode);
                //根据deviceid统一发送
                for (Map.Entry<String, String> entry : deviceGroupMap) {
                    final SendWebSocket webSocket = webSocketMap.get(entry.getKey());
                    webSocket.sendMessage(JSON.toJSONString(map));
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
